#include <stdio.h>
#include "app/Application.h"   // Be's BApplication class, necessary for any GUI stuff
#include "interface/Window.h"  // Be's BWindow class
#include "interface/View.h"    // Be's BView class
#include "interface/Button.h"  // Be's BButton class
#include "interface/Menu.h"      // Be's BMenu class
#include "interface/MenuItem.h"  // Be's BMenuItem class
#include "interface/MenuBar.h"   // Be's BMenuBar class
#include "interface/Alert.h"     // Be's BAlert class

const uint32 COMMAND_LAUNCH_NUKES   = 'Lnch';
const uint32 COMMAND_LAUNCH_BUNNIES = 'LncB';
const uint32 COMMAND_SHOW_ABOUT_BOX = 'Abot';

/* My custom BWindow subclass... */
class NukesWindow : public BWindow
{
public:
   NukesWindow();
   
   virtual void MessageReceived(BMessage * msg);
   virtual bool QuitRequested();
};

NukesWindow :: NukesWindow()
   : BWindow(BRect(50,50,250,150), "Button BWindow!", B_DOCUMENT_WINDOW, 0)
{
   // This is the horizontal grey menu bar that goes across the top of the window
   BMenuBar * bar = new BMenuBar(BRect(), NULL);
   AddChild(bar);

   // It will contain one item, the File menu.
   BMenu * fileMenu = new BMenu("File");
   bar->AddItem(fileMenu);

   // The file menu contains our "About..." entry 
   fileMenu->AddItem(new BMenuItem("About"B_UTF8_ELLIPSIS, new BMessage(COMMAND_SHOW_ABOUT_BOX)));

   // Put a nested submenu into the File menu, just for fun...
   BMenu * launchSubMenu = new BMenu("Launch");
   launchSubMenu->AddItem(new BMenuItem("Nukes", new BMessage(COMMAND_LAUNCH_NUKES)));
   launchSubMenu->AddItem(new BMenuItem("Bunnies", new BMessage(COMMAND_LAUNCH_BUNNIES)));

   fileMenu->AddItem(launchSubMenu);
   fileMenu->AddSeparatorItem();
   fileMenu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED)));
   
   // Fill this window with a nice light green background BView...
   BRect viewBounds(Bounds());
   viewBounds.top = bar->Bounds().Height();
   BView * view = new BView(viewBounds, NULL, B_FOLLOW_ALL_SIDES, 0);
   view->SetViewColor(100,255,0);
   AddChild(view);

   // And put a nice BButton inside the view.
   BRect buttonRect = view->Bounds();
   buttonRect.InsetBy(20,30);  // we'll make the button a bit smaller than the view

   BButton * button = new BButton(buttonRect, NULL, "Launch nukes!", new BMessage(COMMAND_LAUNCH_NUKES), B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE);
   button->SetTarget(this);  // so that when the button is pressed, its BMessage is sent to this window.
   view->AddChild(button);
}

// Called whenever our window gets a BMessage.  (e.g. from our button)
void NukesWindow :: MessageReceived(BMessage * msg)
{
   printf("NukesWindow got a message!  The message is:  "); 
   msg->PrintToStream();

   switch(msg->what)
   {
      case COMMAND_LAUNCH_NUKES: 
         printf("\n\nLAUNCH NUKES!!! LAUNCH NUKES!!! LAUNCH NUKES!!!\n\n\n");
      break;

      case COMMAND_LAUNCH_BUNNIES: 
         printf("\n\nLaunch bunnies?????\n\n");
      break;

      case COMMAND_SHOW_ABOUT_BOX:
         // The (NULL) parameter makes this BAlert execute asynchronously,
         // instead of blocking our window until the user clicks a button.
         // The BAlert will delete itself when the user clicks!
         (new BAlert("About this app", "This app is getting really silly.", "True"))->Go(NULL); 
      break;

      default:
         BWindow :: MessageReceived(msg);  // all other types go to the parent class's method.
      break;
   }
}

// Called when the user clicks on the window's close box
bool NukesWindow :: QuitRequested()
{
   be_app->PostMessage(B_QUIT_REQUESTED);  // tell the application to quit too
   return true;  // and indicate that it's okay for us to close
}

// Demonstrates a BWindow with a BView and a BButton.  Compile like this:
// gcc window.cpp -lbe
int main(int argc, char ** argv)
{
   BApplication app("application/x-vnd.Sugoi-sillybuttondemo");  // gotta have this!

   // Create the window...
   BWindow * window = new NukesWindow();
   window->Show();  // starts the window's thread and displays the window

   app.Run();  // Doesn't return until the application quits.  Note that for this
               // demo, it will block even after the window is closed.  You'll
               // probably need to control-C to quit!

   return 0;
}
