/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <gui/point.h>
#include <gui/rect.h>
#include "atheossupport/ConvertMessages.h"

status_t ConvertPortableMessageToMessage(const PortableMessage & from, os::Message & to) 
{
   to.MakeEmpty();
   to.SetCode(from.what);

   type_code type;
   int32 count;
   bool fixedSize;

   PortableMessageFieldNameIterator it = from.GetFieldNameIterator();
   const char * n;
   while((n = it.GetNextFieldName()) != NULL)
   {
      if (from.GetInfo(n, &type, &count, &fixedSize) != B_NO_ERROR) return B_ERROR;
      for (int32 j=0; j<count; j++)
      {
         const void * nextItem;
         size_t itemSize;
         if (from.FindData(n, type, j, &nextItem, &itemSize) != B_NO_ERROR) return B_ERROR;

         // do any necessary translation from the Portable data types to Be data types
         switch(type)
         {
            case B_POINT_TYPE:
            {
               const PortablePoint * p = (const PortablePoint *)nextItem;
               os::Point bpoint(p->x, p->y);
               if (to.AddPoint(n, bpoint) != B_NO_ERROR) return B_ERROR;
            }
            break;

            case B_RECT_TYPE:
            {
               const PortableRect * r = (const PortableRect *)nextItem;
               os::Rect brect(r->left, r->top, r->right, r->bottom);
               if (to.AddRect(n, brect) != B_NO_ERROR) return B_ERROR;
            }
            break;
 
            case B_MESSAGE_TYPE:
            {
               PortableMessageRef * msgRef = (PortableMessageRef *) nextItem;
               os::Message amsg;
               if (msgRef->GetItemPointer() == NULL) return B_ERROR;
               if (ConvertPortableMessageToMessage(*msgRef->GetItemPointer(), amsg) != B_NO_ERROR) return B_ERROR;
               if (to.AddMessage(n, &amsg) != B_NO_ERROR) return B_ERROR;
            }
            break;

            default:
               if (to.AddData(n, type, nextItem, itemSize, fixedSize, count) != B_NO_ERROR) return B_ERROR;
            break;
         }
      }
   }
   return B_NO_ERROR;
}

status_t ConvertMessageToPortableMessage(const os::Message & from, PortableMessage & to) 
{
   to.MakeEmpty();
   to.what = from.GetCode();

   int numNames = from.CountNames();
   for (int32 i=0; i<numNames; i++)
   {
      int type;
      int count;
      std::string name = from.GetName(i);
      if (from.GetNameInfo(name.c_str(), &type, &count) == B_NO_ERROR)
      {
         for (int j=0; j<count; j++)
         {
            const void * nextItem;
            size_t itemSize;
            if (from.FindData(name.c_str(), type, &nextItem, &itemSize, j) != B_NO_ERROR) return B_ERROR;

            // do any necessary translation from the AtheOS data types to Portable data types
            switch(type)
            {
               case os::T_POINT:
               {
                  const os::Point * p = (const os::Point *)nextItem;
                  PortablePoint pPoint(p->x, p->y);
                  if (to.AddPoint(name.c_str(), pPoint) != B_NO_ERROR) return B_ERROR;
               }
               break;

               case os::T_RECT:
               {
                  const os::Rect * r = (const os::Rect *)nextItem;
                  PortableRect pRect(r->left, r->top, r->right, r->bottom);
                  if (to.AddRect(name.c_str(), pRect) != B_NO_ERROR) return B_ERROR;
               }
               break;

               case os::T_MESSAGE:
               {
                  os::Message amsg;
                  if (amsg.Unflatten((const uint8 *)nextItem) != B_NO_ERROR) return B_ERROR;
                  PortableMessage * newMsg = newnothrow PortableMessage;
                  if (newMsg)
                  {
                     PortableMessageRef msgRef(newMsg, NULL, NULL);
                     if (ConvertMessageToPortableMessage(amsg, *newMsg) != B_NO_ERROR) return B_ERROR;
                     if (to.AddMessage(name.c_str(), msgRef) != B_NO_ERROR) return B_ERROR;
                  }
                  else {WARN_OUT_OF_MEMORY; return B_ERROR;}
               }
               break;

               default:
                  if (to.AddData(name.c_str(), type, nextItem, itemSize) != B_NO_ERROR) return B_ERROR;
               break;
            }

         }
      }
   }
   return B_NO_ERROR;
}
