#include <stdio.h>

#include <app/Looper.h>
#include "reflector/StorageReflectConstants.h"
#include "besupport/MessageTransceiverThread.h"

const int DEMO_PORT = 4000;  // which TCP port to use, chosen more or less arbitrarily

// This BLooper subclass is used to process the incoming
// BMessages that our MessageTransceiverThread sends to it
class MyLooper : public BLooper
{
public:
   MyLooper(const char * arg)
   {
      _arg = arg;  // save this for the ReadyToRun() call
      _mtt = NULL; // just in case
   }

   ~MyLooper()
   {
      delete _mtt;  // shut down and get rid of the MessageTransceiverThread
   }
   
   virtual thread_id Run()
   {
      thread_id tid = BLooper::Run();
      if (tid >= 0)
      {
         BMessage connectMsg('conn');     // message we want to see when the connection is made
         BMessage disconnectMsg('disc');  // message we want to see when the connection is broken

         // Startup!  Create our MessageTransceiverThread... that is our
         // interface to MUSCLE and to the Internet.
         _mtt = new MessageTransceiverThread(BMessenger(this));  // have it send replies back to me
         if (strcmp(_arg.Cstr(), "accept") == 0)
         {
            if (_mtt->StartAcceptThread(DEMO_PORT, true, &connectMsg, &disconnectMsg) == B_NO_ERROR)
            {
               printf("Now waiting for incoming an TCP connection on port %i\n", DEMO_PORT);
            }
            else
            {
               printf("Aggh!  StartAcceptThread() failed... net_server needs restarting???\n");
               PostMessage(B_QUIT_REQUESTED);
            }
         }
         else
         {
            if (_mtt->StartConnectThread(_arg.Cstr(), DEMO_PORT, &connectMsg, &disconnectMsg) == B_NO_ERROR)
            {
               printf("Now connecting via TCP to %s (port %i)\n", _arg.Cstr(), DEMO_PORT);
            }
            else
            {
               printf("Aggh!  StartConnectThread() failed... net_server needs restarting???\n");
               PostMessage(B_QUIT_REQUESTED);
            }
         }
      }
      return tid;
   }

   virtual void MessageReceived(BMessage * msg)
   {
      printf("\n\n");
      switch(msg->what)
      {
         case 'conn':
            printf("MUSCLE says the TCP connection has been completed/connected!!\n");
         break;

         case 'disc':
            printf("MUSCLE says the TCP connection has been broken/disconnected!!\n");
         break;

         case PORTABLE_MESSAGES_RECEIVED:
         {
            printf("MUSCLE says it got some PortableMessages from the TCP connection!  They are:\n");
            PortableMessageRef msgRef;
            while(_mtt->GetNextIncomingMessage(msgRef) == B_NO_ERROR)
            {
               PortableMessage * pmsg = msgRef.GetItemPointer();
               pmsg->PrintToStream();
            }
         }
         break;

         default:
            BLooper::MessageReceived(msg);
         break;
      }
   }

   MessageTransceiverThread * GetMessageTransceiverThread() {return _mtt;}

private:
   PortableString _arg;
   MessageTransceiverThread * _mtt;
};

void main(int argc, char ** argv)
{
   if (argc < 2)
   {
      printf("Usage:  directconnect accept     (to accept an incoming connection)\n");
      printf("   or:  directconnect ipaddress  (to make an outgoing connection to ipaddress)\n");
      exit(5);
   }

   // Create and start up our incoming-messages-processing looper
   MyLooper * myLooper = new MyLooper(argv[1]);
   myLooper->Run();
 
   // Now we will just watch stdin for user input.  A program with
   // a GUI would probably create a BApplication and call Run() on
   // that instead, but we can't be bothered with a GUI in this example.
   while(1)
   {
      char text[1000];
      gets(text);
      printf("You typed: [%s]\n",text);
      if (strcmp(text, "quit") == 0)
      {
         printf("Quitting!\n");
         break;
      }
      else
      {
         PortableMessage * msg = new PortableMessage('chat');
         msg->AddString("text", text);

         printf("Sending a PortableMessage out to the MessageTranceiverThread... the message is:\n");
         msg->PrintToStream();

         PortableMessageRef msgRef(msg, NULL, NULL);
         myLooper->GetMessageTransceiverThread()->SendOutgoingMessage(msgRef);
      }
   }

   if (myLooper->Lock()) myLooper->Quit();  // shut down the looper

   printf("Bye bye!\n"); 
   return 0;
}

