/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <unistd.h>

#include "util/PortableQueue.h"
#include "util/PortableString.h"

#define TEST(x) if (!(x)) printf("Test failed, line %i\n",__LINE__)

void PrintToStream(const PortableQueue<int> & q);
void PrintToStream(const PortableQueue<int> & q)
{
   printf("Queue state is:\n");
   for (int i=0; i<q.CountItems(); i++)
   {
/*
      int val;
      TEST(q.GetItemAt(i, val));    
      printf("%i -> %i\n",i,val);
*/
      printf("%i -> %i\n",i,q[i]);
   }
}

int SortInts(const int & i1, const int & i2);
int SortInts(const int & i1, const int & i2)
{
   return i1-i2;
}

int SortStrings(const PortableString & i1, const PortableString & i2);
int SortStrings(const PortableString & i1, const PortableString & i2)
{
   return strcmp(i1(), i2());
}

// This program exercises the PortableQueue class.
void main(void) 
{
   const int testSize = 15;
   PortableQueue<int> q;

   printf("ADDTAIL TEST\n");
   {
      for (int i=0; i<testSize; i++) 
      {
         TEST(q.AddTail(i));
         PrintToStream(q);
      }
   } 

   printf("REPLACEITEMAT TEST\n");
   {
      for (int i=0; i<testSize; i++) 
      {
         TEST(q.ReplaceItemAt(i, i+10));
         PrintToStream(q);
      }
   } 

   printf("INSERTITEMAT TEST\n");
   {
      for (int i=0; i<testSize; i++) 
      {
         TEST(q.InsertItemAt(i,i));
         PrintToStream(q);
      }
   }

   printf("REMOVEITEMAT TEST\n");
   {
      for (int i=0; i<testSize; i++) 
      {
         TEST(q.RemoveItemAt(i));
         PrintToStream(q);
      }
   }

   printf("SORT TEST 1\n");
   {
      q.MakeEmpty();
      for (int i=0; i<testSize; i++)
      {
         int next = rand()%255;
         TEST(q.AddTail(next));
         printf("Added item %i = %i\n", i, q[i]);
      }
      printf("sorting ints...\n");
      q.Sort(SortInts);
      for (int j=0; j<testSize; j++) printf("Now item %i = %i\n", j, q[j]);
   }

   printf("SORT TEST 2\n");
   {
      PortableQueue<PortableString> q2;
      for (int i=0; i<testSize; i++)
      {
         int next = rand()%255;
         char buf[64];
         sprintf(buf, "%i", next);
         TEST(q2.AddTail(buf));
         printf("Added item %i = %s\n", i, q2[i].Cstr());
      }
      printf("sorting strings...\n");
      q2.Sort(SortStrings);
      for (int j=0; j<testSize; j++) printf("Now item %i = %s\n", j, q2[j].Cstr());
   }

   printf("REVERSE TEST\n");
   {
      q.MakeEmpty();
      for (int i=0; i<testSize; i++) TEST(q.AddTail(i));
      q.ReverseItemOrdering();
      for (int j=0; j<testSize; j++) printf("After reverse, %i->%i\n", j, q[j]);
   }
}
